"""
CellRendererMessage.py
by Jason Conti
February 23, 2010

A cell renderer to display notifications.

Based on ideas from witterCellRenderer.py:
  http://danielwould.wordpress.com/witter/
"""

import cairo
import gobject
import gtk
import pango
import pangocairo

class CellRendererMessage(gtk.GenericCellRenderer):
  """A cell renderer to display notifications."""
  __gproperties__ = {
      "message": (gobject.TYPE_OBJECT,
          "message to display",
          "message to display",
          gobject.PARAM_READWRITE)
  }
  def __init__(self):
    gtk.GenericCellRenderer.__init__(self)
    self.__properties = {}

  def on_get_size(self, widget, cell_area):
    message = self.get_property("message")
    w = 400
    h = 20

    if message != None:
      surface = cairo.ImageSurface(cairo.FORMAT_ARGB32, w, w)
      cr = pangocairo.CairoContext(cairo.Context(surface))

      layout = self.create_summary_layout(cr, message.summary, w)
      h += self.get_layout_height(layout)

      layout = self.create_body_layout(cr, message.body, w)
      h += self.get_layout_height(layout)

      layout = self.create_timestamp_layout(cr, message.get_timestamp_line(), w)
      h += self.get_layout_height(layout)

    if cell_area == None:
      return (0, 0, w, h)
    else:
      return (cell_area.x, cell_area.y, w, h)

  def get_layout_height(self, layout):
    return layout.get_pixel_extents()[1][3]

  def create_summary_layout(self, cr, summary, width):
    layout = cr.create_layout()
    font = pango.FontDescription("Sans")
    font.set_size(pango.SCALE * 10)
    font.set_style(pango.STYLE_NORMAL)
    font.set_weight(pango.WEIGHT_BOLD)
    layout.set_font_description(font)
    layout.set_width(pango.SCALE * width)
    layout.set_wrap(pango.WRAP_WORD)
    layout.set_markup(summary)
    return layout

  def create_body_layout(self, cr, body, width):
    layout = cr.create_layout()
    font = pango.FontDescription("Sans")
    font.set_size(pango.SCALE * 10)
    font.set_style(pango.STYLE_NORMAL)
    font.set_weight(pango.WEIGHT_NORMAL)
    layout.set_font_description(font)
    layout.set_width(pango.SCALE * width)
    layout.set_wrap(pango.WRAP_WORD)
    layout.set_markup(body)
    return layout

  def create_timestamp_layout(self, cr, timestamp, width):
    layout = cr.create_layout()
    font = pango.FontDescription("Sans")
    font.set_size(pango.SCALE * 8)
    font.set_style(pango.STYLE_NORMAL)
    font.set_weight(pango.WEIGHT_NORMAL)
    layout.set_font_description(font)
    layout.set_width(pango.SCALE * width)
    layout.set_wrap(pango.WRAP_WORD)
    layout.set_markup(timestamp)
    return layout

  def do_set_property(self, key, value):
    self.__properties[key] = value

  def do_get_property(self, key):
    return self.__properties[key]

  def on_render(self, window, widget, background_area, cell_area, expose_area, flags):
    cairo_context = pangocairo.CairoContext(window.cairo_create())

    x = cell_area.x
    y = cell_area.y
    w = cell_area.width
    h = cell_area.height

    self.render_rect(cairo_context, x, y, w, h)

    pattern = cairo.SolidPattern(0.15, 0.15, 0.15, 1)
    cairo_context.set_source(pattern)
    cairo_context.fill()

    message = self.get_property("message")
    if message == None:
      return

    cairo_context.set_source_rgba(1, 1, 1, 1)

    x0 = x + 5
    y0 = y + 5

    layout = self.create_summary_layout(cairo_context, message.summary, w - 40)
    summary_h = self.get_layout_height(layout)
    cairo_context.move_to(x0, y0)
    cairo_context.show_layout(layout)

    y0 += summary_h + 5
    
    layout = self.create_body_layout(cairo_context, message.body, w - 40)
    body_h = self.get_layout_height(layout)
    cairo_context.move_to(x0, y0)
    cairo_context.show_layout(layout)

    y0 += body_h + 5

    layout = self.create_timestamp_layout(cairo_context, message.get_timestamp_line(), w - 10)
    cairo_context.move_to(x0, y0)
    cairo_context.show_layout(layout)

  def render_rect(self, cr, x, y, w, h):
    x0 = x
    y0 = y
    x1 = x + w
    y1 = y + h

    radius = 5

    cr.move_to(x0, y0 + radius)
    cr.curve_to(x0, y0 + radius, x0, y0, x0 + radius, y0)
    cr.line_to(x1 - radius, y0)
    cr.curve_to(x1 - radius, y0, x1, y0, x1, y0 + radius)
    cr.line_to(x1, y1 - radius)
    cr.curve_to(x1, y1 - radius, x1, y1, x1 - radius, y1)
    cr.line_to(x0 + radius, y1)
    cr.curve_to(x0 + radius, y1, x0, y1, x0, y1 - radius)
    cr.close_path()

  def on_activate(self, event, widget, path, background_area, cell_area, flags):
    pass

  def on_start_editing(self, event, widget, path, background_area, cell_area, flags):
    pass
